# Copyright 2005, Rick Wilson 

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

# Name :          pathCopy
# Description :   Copies a selected group/component along a selected path
# Author :        Rick Wilson
# Usage :         1. Install into the plugins directory (or subfolder - supports organize.rb) or into the 
#                    Plugins/examples directory and manually load from the ruby console "load 'examples/bomb.rb'" 
#                 2. Run "Copy along path" from the Plugins menu(or organized submenu).
# Date :          2005-11-29
# Type :          Tool
# History:
#			2.0 (????-??-??) - planned improvements:
#							Specify distance between copies
#							Specify divisor
#			1.0 (2005-11-29) - first version - copies selected component to each vertex of a curve
# 

require 'sketchup.rb'

class PathCopy

def initialize
	@path=nil
end

def activate
	puts "activating..."
	sel=Sketchup.active_model.selection
	if sel.length>0
		if sel.first.typename=="Edge"
			if sel.first.curve
				@path=sel.first.curve
				puts "path selected"
			else
				@path=sel.first
				puts "path selected"
			end
		end
	    Sketchup::set_status_text($exStrings.GetString("Select group/component to copy"), SB_PROMPT)
	else
	    Sketchup::set_status_text($exStrings.GetString("Select path"), SB_PROMPT)
	end
end

def onLButtonDown(flags,x,y,view)
	ph=view.pick_helper
	num=ph.do_pick x,y
	item=ph.best_picked
	puts item

	if item
		get_path(item) if @path==nil
		pathCopyComponent(item) if item.typename=="ComponentInstance"
		pathCopyGroup(item) if item.typename=="Group"
	end
end

def get_path(item)
	if item
		if item.typename=="Edge"
			if item.curve
				@path=item.curve
				puts "path selected"
			else
				@path=item
				puts "path selected"
			end
			Sketchup::set_status_text($exStrings.GetString("Select group/component to copy"), SB_PROMPT)
		end
	end
	Sketchup.active_model.selection.clear
	Sketchup.active_model.selection.add(@path.edges) if @path.typename=="Curve"|| @path.typename=="ArcCurve"
	Sketchup.active_model.selection.add(@path) if @path.typename=="Edge"
end

def pathCopyComponent(item)
	Sketchup.active_model.selection.clear
	model=Sketchup.active_model
	ents=model.active_entities
	puts "Copy component instance along path"
	points=@path.vertices
	points.collect!{|p| p.position}
	model.start_operation "pathCopy"
	0.upto(points.length-1) do |i|
		t=Geom::Transformation.new(points[i])
		ents.add_instance(item.definition,t)
	end
	model.commit_operation
	@path=nil
	Sketchup::set_status_text($exStrings.GetString("Select path"), SB_PROMPT)
end

def pathCopyGroup(item)
	Sketchup.active_model.selection.clear
	model=Sketchup.active_model
	ents=model.active_entities
	puts "Copy group along path"
	points=@path.vertices
	points.collect!{|p| p.position}
	model.start_operation "pathCopy"
	0.upto(points.length-1) do |i|
		t1=Geom::Point3d.new(0,0,0)-item.transformation.origin
		t2=Geom::Transformation.new(points[i])
		gp=item.copy
		gp.transformation=item.transformation
		gp.transform!(t1)
		gp.transform!(t2)
	end
	model.commit_operation
	@path=nil
	Sketchup::set_status_text($exStrings.GetString("Select path"), SB_PROMPT)
end

def deactivate(view)
	view.invalidate if @drawn
end

def onCancel(flag, view)
	self.reset(view)
end

def reset(view)
	@path=nil
	Sketchup::set_status_text($exStrings.GetString("Select path"), SB_PROMPT)
	if( view )
		view.tooltip = nil
		view.invalidate
	end
	
end



end #class

if( not file_loaded?("pathcopy.rb") )
# BEGIN CHANGES BY organizerEdit.rb
if $submenu!=nil
$submenu.add_item("Kopieren auf Pfad") { Sketchup.active_model.select_tool PathCopy.new }
else
UI.menu("Plugins").add_item("Kopieren auf Pfad") { Sketchup.active_model.select_tool PathCopy.new }
end
# END CHANGES BY organizerEdit.rb
end
#-----------------------------------------------------------------------------
file_loaded("pathcopy.rb")
